/*
 * CORE10GMAC_tb.v
 *
 * Description:
 *    Visual testbench for CORE10GMAC
 *
 */

`define TAMBA_MODEL_ASYNC_FIFO_READ_DLY
`define TAMBA_TECHNOLOGY_TARGET_MICROSEMI

`timescale 1ps/1ps

module testbench;

     /* Examples
      */
   parameter TYPE_10G = 1; // currently supports 1 or 2 
   //parameter TB_C36 = 1; // Temporary disabled
   parameter TB_C49 = 1;
   parameter TB_C72 = 1;
   parameter TB_C73 = 1;

   parameter TB_C72_CNT  = 6;
   
     /* personality 
      */
   parameter SWC  = (TYPE_10G == 2) ? 2: 1;
   parameter CWC  = SWC;
   parameter PWC  = 2;
   parameter FAMILY = 0;
     /* personality, constants
      */
   parameter BPW  = 4;
   
     /* personality, derived
      */
   parameter SBC  = SWC*BPW;
   parameter CBC  = CWC*BPW;
   parameter PBC  = PWC*BPW;
   
     /* performance
      */
   parameter CFG_SILICON_TARGET    = "MICROSEMI";
   parameter CFG_SILICON_FAMILY    = "?";
   parameter CFG_SILICON_SPEED     = 0;
   
     /* features, mac
      */
   parameter CFG_MAC_TX_FIFO_DEPTH      = 32;
   parameter CFG_MAC_RX_FIFO_DEPTH      = 32;

     /* features, pcs36
      */
   //parameter CFG_PCS36_EN            = 1;
   
     /* Clocks Periods
      */
   parameter CLK_PERIOD_100M  = 10000;
   parameter CLK_PERIOD_200M  = 5000;
   parameter CLK_PERIOD_156M  =  6400;
   parameter CLK_PERIOD_322M  =  3100;
   
      /* features, pcs73
      */
   parameter CFG_PCS73_RX_GRBX_EN                    = 1;
   
     /* Clocks Periods, derived
      */
   parameter CLK_PERIOD_SYS   = ((SWC==1) ? CLK_PERIOD_322M : CLK_PERIOD_156M);
   parameter CLK_PERIOD_CORE  =             CLK_PERIOD_322M;
   
   parameter i_cfg_crc              = (TYPE_10G == 2) ? 1'b0 : 1'b1;
/*************************************************************************************
 * register and wire declarations
 * 
 *************************************************************************************/
 
   reg [31:0] DATA_TEST_WR;
   reg [31:0] DATA_TEST_RD;
   
   
   reg             clk156m;
   reg             clk322m;
   reg             clk100m;
   reg 		       clk200m;
   wire            sys_clk;
   wire            core_tx_clk;
   wire            core_rx_clk;
      
   reg             sys_sreset;
   wire            core_tx_sreset;
   wire            core_rx_sreset;
         
     /* MAC Tx
      */
   wire             mtx_rdy;
   wire             mtx_fault;
   wire             mtx_fifo_af;
   
   wire             mtx_en;
   wire             mtx_sop;
   wire             mtx_eop;
   wire [7:0]       mtx_bc;
   wire [SBC*8-1:0] mtx_data;
   
     /* MAC Rx
      */
   wire             mrx_rdy;
   wire             mrx_en;
   wire             mrx_sop;
   wire             mrx_eop;
   wire             mrx_err;
   wire [7:0]       mrx_bc;
   wire [SBC*8-1:0] mrx_data;
   
     /* RS
      */
   wire            rs_rx_fault_local;
   wire            rs_rx_fault_remote;
   
     /* PCS
      */
   wire            pcs49_rx_block_lock;
   wire            pcs49_rx_hi_ber;
   wire            pcs49_rx_status;
   
   wire            pcs36_rx_status;
   
     /* CPU
      */
   wire            cpu_clk;
   reg             cpu_sreset;

     /* APB
      */
   wire         PCLK;
   wire         PRESETN;
   reg          PWRITE;
   reg  [15:0]  PADDR;
   reg          PSEL;
   reg          PENABLE;
   reg  [31:0]  PWDATA;
   wire [31:0]  PRDATA;
   wire         PREADY;
   wire         PSLVERR;
   
   /* XGMII
   */
   wire [CWC*32-1:0]     O_XGMII_TXD;
   wire [CWC*4-1:0]      O_XGMII_TXC;

     /* PMA
      */
   wire [CBC*8-1:0]    pma_tx_raw_data;

     /* c72_
      */
   reg [31:0]             c72_tx_page;
   reg [15:0]             c72_tx_cnt;
   reg                    c72_rx_page_rdy;
   reg [31:0]             c72_rx_page;
   reg                    c72_rx_error;
   
     /* c73_
      */
   reg [47:0]         c73_tx_page;
   
   reg                c73_rx_page_rdy;
   reg [47:0]         c73_rx_page;
   reg                c73_rx_error;

     /* Sim
      */
   reg [15:0]            cfg_size_min;
   reg [15:0]            cfg_size_max;
   reg [15:0]            cfg_rep_r;
   reg [7:0]             cfg_ipg_r;
   
   reg             start_strb_r;
   wire            mtx_first_strb;
   wire            mtx_end_strb;
   wire            mrx_first_strb;
   wire            mrx_end_strb;
   wire            mrx_end_ok;
   
   reg 		       rx_1st_r;

   reg             bypass_scrm;

   reg [31:0]      task_rdata;

   
   integer         i;
   
   
/*************************************************************************************
 * module, m_dut
 * 
 *************************************************************************************/
   CORE10GMAC #
     (
        /* personality
     */
      .TYPE_10G(TYPE_10G),
      .SWC(SWC),
      .CWC(CWC),
      
        /* period
     */
      .CLK_PERIOD_SYS   (CLK_PERIOD_SYS),
      .CLK_PERIOD_CORE  (CLK_PERIOD_CORE),

        /* performance
     */
      .CFG_SILICON_TARGET  (CFG_SILICON_TARGET),
      .CFG_SILICON_FAMILY  (CFG_SILICON_FAMILY),
      .CFG_SILICON_SPEED   (CFG_SILICON_SPEED),
	  
        /* features, pcs73
     */
      .CFG_PCS73_RX_GRBX_EN(CFG_PCS73_RX_GRBX_EN),
	  .PCS73_LPBK_SIM(1),                           // PCS73 Loopback - Only used for simulation, tied to 1
      
        /* features, mac
     */
      .CFG_MAC_TX_FIFO_DEPTH      (CFG_MAC_TX_FIFO_DEPTH),
      .CFG_MAC_RX_FIFO_DEPTH      (CFG_MAC_RX_FIFO_DEPTH)//,
        /* features, pcs36
     */
      //.CFG_PCS36_EN            (CFG_PCS36_EN)
      )
     m_dut
       (
      /* System
       */
         .I_SYS_CLK                                 (sys_clk),             
         .I_SYS_TX_SRESET                           (sys_sreset),          
        .I_SYS_RX_SRESET                           (sys_sreset),          
    
      /* Core                                                             
       */                                                                 
         .I_CORE_TX_CLK                              (core_tx_clk),        
        .O_CORE_TX_SRESET                           (core_tx_sreset),
    
         .I_CORE_RX_CLK                             (core_rx_clk),         
        .O_CORE_RX_SRESET                          (core_rx_sreset),              
    
      /* MAC, Pkt Intf, tx                                                
       */                                                                 
         .O_SYS_MAC_TX_RDY                          (mtx_rdy),             
         .O_SYS_MAC_TX_FAULT                        (mtx_fault),           
         .O_SYS_MAC_TX_FIFO_AF                      (mtx_fifo_af),         
    
         .I_SYS_MAC_TX_EN                           (mtx_en),              
         .I_SYS_MAC_TX_SOP                          (mtx_sop),             
         .I_SYS_MAC_TX_EOP                          (mtx_eop),                
         .I_SYS_MAC_TX_BC                           (mtx_bc),              
         .I_SYS_MAC_TX_DATA                         (mtx_data),    
         .O_SYS_MAC_TX_ERR_BUS_PROTOCOL             (/*nc*/),              
         .O_SYS_MAC_TX_ERR_FIFO_OVERFLOW            (/*nc*/),              
         .O_SYS_MAC_TX_ERR_FIFO_UNDERRUN            (/*nc*/),              
         .O_SYS_MAC_TX_STATS_VECTOR                 (/*nc*/),      

          /* MAC, Lpbk
           */
         .I_MAC_RX_LPBK_LOCAL_CLK                   (1'b0),    
         .I_MAC_RX_LPBK_LOCAL_CALL                  (32'b0),   
         .I_MAC_RX_LPBK_LOCAL_CTRL_W                ({CWC*32{1'b0}}),   
         .I_MAC_RX_LPBK_LOCAL_DATA_W                ({CWC*BPW*8{1'b0}}),          
            
         .O_MAC_TX_LPBK_LOCAL_CLK                   (/*nc*/),    
         .O_MAC_TX_LPBK_LOCAL_CALL                  (/*nc*/),   
         .O_MAC_TX_LPBK_LOCAL_CTRL_W                (/*nc*/),   
         .O_MAC_TX_LPBK_LOCAL_DATA_W                (/*nc*/),           
    
       /* MAC, Pkt Intf, rx                                               
        */                              
         .I_SYS_MAC_RX_GFC                          (1'b0),
         .O_SYS_MAC_RX_RDY                          (mrx_rdy),              
         .O_SYS_MAC_RX_EN                           (mrx_en),              
         .O_SYS_MAC_RX_SOP                          (mrx_sop),             
         .O_SYS_MAC_RX_EOP                          (mrx_eop),             
         .O_SYS_MAC_RX_ERR                          (mrx_err),             
         .O_SYS_MAC_RX_ERR_W                        (/*nc*/),              
         .O_SYS_MAC_RX_BC                           (mrx_bc),              
         .O_SYS_MAC_RX_DATA                         (mrx_data),            
         .O_SYS_MAC_RX_ERR_FIFO_OVERFLOW            (mrx_overflow),    
         .O_SYS_MAC_RX_PREAMBLE                     (/*nc*/), 
         .O_SYS_MAC_RX_STATS_VECTOR                 (/*nc*/),              
    
      /* Pause, tx                                                        
       */                                                                 
         .I_CFG_PAUSE_TX_SEND_STRB                  (1'b0),                
         .I_CFG_PAUSE_TX_BEAT_EN                    (1'b0),                
         .I_CFG_PAUSE_TX_BEAT_TIME                  (16'h0),               
         .I_CFG_PAUSE_TX_PORT_EN                    (1'b0),                
         .I_CFG_PAUSE_TX_PORT_TIME                  (16'h0),               
         .I_CFG_PAUSE_TX_PFC_EN_W                   (8'h0),                
         .I_CFG_PAUSE_TX_PFC_TIME_W                 (128'h0),              
         .I_PAUSE_TX_PORT_XOFF                      (1'b0),                
         .I_PAUSE_TX_PFC_XOFF_W                     (8'h0),                
         .I_PAUSE_TX_PORT_REQ                       (1'b0),                
         .O_PAUSE_TX_PORT_ON                        (/*nc*/),              
    
      /* Pause, rx                                                        
       */                                                                 
         .I_SYS_PAUSE_RX_SRESET                     (core_rx_sreset),      
         .I_CFG_SYS_PAUSE_RX_PORT_EN                (1'b0),                
         .I_CFG_SYS_PAUSE_RX_PFC_EN_W               (8'h0),                
         .O_SYS_PAUSE_RX_PORT_XOFF                  (/*nc*/),              
         .O_SYS_PAUSE_RX_PFC_XOFF_W                 (/*nc*/),              
    
      /* RS, tx                                                           
       */                                                                 
         .I_CFG_RS_TX_FAULT_EN                      (1'b0),                
         .I_CFG_RS_TX_FAULT_LOCAL                   (1'b0),                
         .I_CFG_RS_TX_FAULT_REMOTE                  (1'b0),                
         .I_CFG_RS_TX_IDLE                          (1'b0),                
    
      /* RS, rx                                                           
       */                                                                 
         .I_RS_RX_SRESET                            (core_rx_sreset),      
         .O_RS_RX_FAULT_LOCAL                       (rs_rx_fault_local),   
         .O_RS_RX_FAULT_REMOTE                      (rs_rx_fault_remote),  
    
      /* PTP, tx                                                          
       */                                                                 
         .I_PTP_TX_TOD_REF                          (96'h0),     
         .I_SYS_PTP_TX_1STEP_TIMESTAMP_EN           (1'b0),      
    
      /* PTP, rx                                                          
       */                                                                  
         .I_PTP_RX_TOD_REF                          (96'h0),    
         .O_SYS_PTP_RX_EN                           (/*nc*/),
    
      /* PCS, c49, tx                                                     
       */                                                                 
         .I_PCS49_TX_SRESET                         (core_tx_sreset),      
         .I_CFG_PCS49_TX_BYPASS_SCRAMBLER           (bypass_scrm),         
         .I_CFG_PCS49_TX_TEST_PRBS31_EN             (1'b0),                
         .I_CFG_PCS49_TX_TEST_PATTERN_EN            (1'b0),                
         .I_CFG_PCS49_TX_TEST_PATTERN_TYPE_SEL      (1'b0),                
         .I_CFG_PCS49_TX_TEST_PATTERN_DATA_SEL      (1'b0),                
         .I_CFG_PCS49_TX_TEST_PATTERN_SEED_A        (58'h0),               
         .I_CFG_PCS49_TX_TEST_PATTERN_SEED_B        (58'h0),               
    
      /* PCS, c49, rx                                                 
       */                                                                 
        .I_PCS49_RX_SRESET                         (core_rx_sreset),      
        .I_CFG_PCS49_RX_BYPASS_SCRAMBLER           (bypass_scrm),         
        .I_CFG_PCS49_RX_TEST_PRBS31_EN             (1'b0),                
        .I_CFG_PCS49_RX_TEST_PATTERN_EN            (1'b0),                
        .I_CFG_PCS49_RX_TEST_PATTERN_TYPE_SEL      (1'b0),                
        .I_CFG_PCS49_RX_TEST_PATTERN_DATA_SEL      (1'b0),                
    
      /* PCS, c49, rx Status & Stats                                  
       */                                                                 
        .O_PCS49_RX_BLOCK_LOCK                     (pcs49_rx_block_lock), 
        .O_PCS49_RX_HI_BER                         (pcs49_rx_hi_ber),     
        .O_PCS49_RX_STATUS                         (pcs49_rx_status),     
        .O_PCS49_RX_BER_STRB                       (/*nc*/),              
        .O_PCS49_RX_BER_CNT                        (/*nc*/),              
        .O_PCS49_RX_TEST_MODE_ERR_STRB             (/*nc*/),              
        .O_PCS49_RX_TEST_MODE_ERR_CNT              (/*nc*/),              
        .O_PCS49_RX_ERRORED_BLOCK_CNT_STRB         (/*nc*/),              
    
      /* PCS, c36, tx                                                     
       */                                                                 
        .I_PCS36_TX_SRESET                         (core_tx_sreset),      
    
      /* PCS, c36, rx                                                     
       */                                                                 
        .I_PCS36_RX_SRESET                         (core_rx_sreset),      
        .O_PCS36_RX_STATUS                         (pcs36_rx_status),     
        .O_PCS36_RX_ERR_NOTINTABLE                 (/*nc*/),              
        .O_PCS36_RX_ERR_DISPERR                    (/*nc*/),           
        .O_PMA_RX_RAW_BIT_SLIP                     (/*nc*/),            
    
      /* Clause c72 & c73                                                 
       */                                                    
        .PCLK                                      (PCLK),
        .PRESETN                                   (PRESETN),
        .PWRITE                                    (PWRITE),
        .PADDR                                     (PADDR),
        .PSEL                                      (PSEL),
        .PENABLE                                   (PENABLE),
        .PWDATA                                    (PWDATA),
        .PRDATA                                    (PRDATA),
        .PREADY                                    (PREADY),
        .PSLVERR                                   (PSLVERR),

      /* PMA
       */
        .O_PMA_TX_RAW_DATA                         (pma_tx_raw_data),
    
        .I_PMA_RX_RAW_RDY                          (1'b1),
        .I_PMA_RX_RAW_EN                           (1'b1),
        .I_PMA_RX_RAW_DATA                         (pma_tx_raw_data),

      /* C49 TX GEARBOX
      */
        .O_PMA49_TX_GRBX_SOS                       (/*nc*/),    
        .O_PMA49_TX_GRBX_HDR_EN                    (/*nc*/),    
        .O_PMA49_TX_GRBX_HDR                       (/*nc*/),      
        .O_PMA49_TX_GRBX_DATA_EN                   (/*nc*/),  
        .O_PMA49_TX_GRBX_DATA                      (/*nc*/),     
      
      /* C49 RX GEARBOX
      */
        .I_PMA49_RX_GRBX_LOCK                      (1'b0),   
        .I_PMA49_RX_GRBX_SOS                       (1'b0),       
        .I_PMA49_RX_GRBX_HDR_EN                    (1'b0),    
        .I_PMA49_RX_GRBX_HDR                       (4'b0),       
        .I_PMA49_RX_GRBX_DATA_EN                   (1'b0),   
        .I_PMA49_RX_GRBX_DATA                      ({SWC*32{1'b0}}),
      /* XMGII
      */
        .O_XGMII_TXD            (O_XGMII_TXD),
        .O_XGMII_TXC            (O_XGMII_TXC),
        .I_XGMII_RXD            (O_XGMII_TXD),
        .I_XGMII_RXC            (O_XGMII_TXC),
      /* pcs73
	  */  
        .I_PCS73_VAL           (!sys_sreset) ,
        .I_PCS73_RX_CLK        (core_rx_clk) ,
        .I_PCS73_TX_CLK        (core_tx_clk) ,
        .I_PCS73_DATA          ({(CWC*32){1'b0}})
                                                    
    ); 

/*************************************************************************************
 * Stream Tx
 * 
 *************************************************************************************/
   tamba_model_tx_intf_enet #
     (
      .SBC  (SBC),
      .PWC  (PWC),
      .BPW  (BPW)
      )
     model_tx
       (
        .i_clk           (sys_clk),
         .i_sreset        (sys_sreset),

      /* Cfg
       */
        .i_cfg_size_min  (cfg_size_min),
        .i_cfg_size_max  (cfg_size_max),
        .i_cfg_rep       (cfg_rep_r),
        .i_cfg_ipg       (cfg_ipg_r),
        .i_cfg_pad       (1'b0),
        .i_cfg_pad_size  (8'd64),
        .i_cfg_crc       (i_cfg_crc),
            
      /* Control
       */
         .i_start_strb    (start_strb_r),
         .o_first_strb    (mtx_first_strb),
         .o_end_strb      (mtx_end_strb),

      /* Stream Interface
       */
         .i_fc            (mtx_fifo_af),
    
        .o_en            (mtx_en),
        .o_sop           (mtx_sop),
        .o_eop           (mtx_eop),
        .o_eope          (/*nc*/),
        .o_bc            (mtx_bc),
        .o_preamble      (/*nc*/),
        .o_data          (mtx_data)
    ); 

/*************************************************************************************
 * Stream Rx
 * 
 *************************************************************************************/
   tamba_model_rx_intf_enet #
     (
      .SBC  (SBC),
      .PWC  (PWC),
      .BPW  (BPW)
      )
     model_rx
       (
         .i_clk              (sys_clk),
         .i_sreset           (sys_sreset),

      /* Configuration
       */
        .i_cfg_size_min     (cfg_size_min),
        .i_cfg_size_max     (cfg_size_max),
        .i_cfg_rep          (cfg_rep_r),
        .i_cfg_preamble_en  (1'b0),
        .i_cfg_preamble_wc  (3'h1),
        .i_cfg_pad          (1'b1),
        .i_cfg_pad_size     (8'd64),
        .i_cfg_crc          (1'b1),
            
      /* Control
       */
        .i_start_strb       (start_strb_r),
        .o_first_strb       (mrx_first_strb),
        .o_end_strb         (mrx_end_strb),
        .o_end_ok           (mrx_end_ok),

      /* Stream Interface
       */
        .i_rx_en            (mrx_en),
        .i_rx_sop           (mrx_sop),
        .i_rx_eop           (mrx_eop),
        .i_rx_err           (mrx_err),
        .i_rx_bc            (mrx_bc),
        .i_rx_preamble      ({PBC{8'h0}}),
        .i_rx_data          (mrx_data)
    ); 

/*************************************************************************************
 * Clocks
 * 
 *************************************************************************************/
   initial
     begin
       clk100m  = 1'b0;
       clk200m  = 1'b0;
       clk156m  = 1'b0;
       clk322m  = 1'b0;
       DATA_TEST_WR = 32'hFFFFFFFF;
     end

   always
     #(CLK_PERIOD_100M/2) clk100m <= ~clk100m;
	 
   always
     #(CLK_PERIOD_200M/2) clk200m <= ~clk200m;

   always
     #(CLK_PERIOD_156M/2) clk156m <= ~clk156m;

   always
     #(CLK_PERIOD_322M/2) clk322m  <= ~clk322m;

   assign  sys_clk      = ((SWC==1) ? clk322m : clk156m);
   assign  core_tx_clk  = ((SWC==1) ? clk322m : clk156m);
   assign  core_rx_clk  = ((SWC==1) ? clk322m : clk156m);
   assign  cpu_clk      =             clk100m;
   assign  PCLK         =             clk100m;
   assign  PRESETN      =             !cpu_sreset; 
   
/*************************************************************************************
 * Simulation
 * 
 *************************************************************************************/
initial 
  begin

    if(TYPE_10G == 1) 
      begin
        $display("\n");
        $display("\n");
        $display("TestBench (Visual) for Core10GMAC \n");
          
        cfg_size_min  = 16'd60;
        cfg_size_max  = 16'd73;
        
        cfg_rep_r     = 16'd1;
        cfg_ipg_r     =  8'd0;
            
        start_strb_r  =  1'b0;
            
        sys_sreset    =  1'b1;
        cpu_sreset    =  1'b1;
        
        bypass_scrm   =  1'b0;
       
          /* CPU Reset
           */
        repeat (32)
          @(posedge cpu_clk);
          cpu_sreset = 1'b0;
        @(posedge cpu_clk);
       
          /* Sys Reset
           */
        repeat (32)
          @(posedge sys_clk);
          sys_sreset = 1'b0;
        @(posedge sys_clk);
        
        wait (mtx_rdy == 1);
        //if (TB_C36==1) // Temporary disabled
        //    begin
        //       $display;
        //       $display("PCS Clause 36");
        //
        //     // Enable tx/rx pcs36
        //       cpu_write ((16'h40<<2),32'h01);
        //       cpu_write ((16'h50<<2),32'h01); 
        //       
        //   // Set pma_tx_sel to 1
        //       cpu_write ((16'h80<<2),32'h01);  
        //       
        //         /* Sys Reset
        //      */
        //       repeat (32)
        //       @(posedge sys_clk);
        //       sys_sreset = 1'b0;
        //       @(posedge sys_clk);
        //       
        //         /* wait
        //      */
        //       $display("Waiting for PCS Status");
        //       while (!pcs36_rx_status)
        //       @(posedge sys_clk);
        //       $display("PCS Status ok");
        //
        //         /* Start
        //      */
        //       start_strb_r  = 1'b1;
        //       @(posedge sys_clk);
        //       start_strb_r  = 1'b0;
        //       @(posedge sys_clk);
        //       
        //       while (!mrx_end_strb)
        //       @(posedge sys_clk);
        //       
        //       repeat (32)
        //       @(posedge sys_clk);
        //    end
               
        if (TB_C49==1)
            begin
               $display;
               $display("PCS Clause 49");
        
             // Disable tx/rx pcs36
               cpu_write ((16'h40<<2),32'h00); 
               cpu_write ((16'h50<<2),32'h00); 
               
           // Set pma_tx_sel to 0
               cpu_write ((16'h80<<2),32'h00); 
               
                 /* Sys Reset
              */
               repeat (32)
               @(posedge sys_clk);
               sys_sreset = 1'b0;
               @(posedge sys_clk);
               
                 /* wait
              */
               $display("Waiting for PCS Status");
               while (!pcs49_rx_status)
               @(posedge sys_clk);
               $display("PCS Status ok");
               
               repeat (8)
               @(posedge sys_clk);
               
               while (!mtx_rdy)
               @(posedge sys_clk);
               
               $display("Waiting for RS");
               while (rs_rx_fault_local | rs_rx_fault_remote | mtx_fault)
               @(posedge sys_clk);
               $display("RS Status ok");
               
               bypass_scrm  = 1'b1;
        
                 /* wait for transmit RS to clear
              */
               repeat (32)
               @(posedge sys_clk);
               
                 /* Start
              */
               start_strb_r  = 1'b1;
               @(posedge sys_clk);
               start_strb_r  = 1'b0;
               @(posedge sys_clk);
               
               while (!mrx_end_strb)
               @(posedge sys_clk);
               
               repeat (32)
               @(posedge sys_clk);
            end
               
        if (TB_C72==1)
          begin
             $display;
             $display("PCS Clause 72");
        
           // Set pma_tx_sel to 3
             cpu_write ((16'h80<<2),32'h03); 
             
             c72_tx_page   = 32'hab896745;
             c72_tx_cnt    = 1'h0;
             c72_rx_error  = 1'b0;
        
             repeat (TB_C72_CNT)
               begin
                /* check boundary
                 */
              case (c72_tx_cnt)
                1:  c72_tx_page = 32'h1;
                2:  c72_tx_page = 32'h0;
                3:  c72_tx_page = 32'h8000_0000;
                4:  c72_tx_page = 32'h0;
              endcase           
        
                /* write tx page
                 */
              for (i=0;i<4;i=i+1)
                cpu_write(((16'h24+i)<<2),{24'b0, c72_tx_page[i*8+:8]});
              
                /* write tx control register
                 [6] - enable tx
                 [0] - send page (continuously)
                 */
                cpu_write((16'h20<<2),32'h41);
              
              $display("Page Transmitted; %h",c72_tx_page);
              
                /* write rx control register
                 [6]  - enable rx
                 [5]  - enable irq
                 */
                cpu_write((16'h30<<2),32'h60);
              
                /* wait for page to be received
                 */
              $display("Waiting for page to be received");
              
              c72_rx_page_rdy  = 1'b0;
              
              while (!c72_rx_page_rdy)
                begin
                   cpu_read((16'h30<<2),task_rdata);
                   c72_rx_page_rdy  = task_rdata[0];
                end
              
                /* read page received
                 */
              for (i=0;i<4;i=i+1)
                cpu_read(((16'h34+i)<<2),c72_rx_page[i*8+:8]);
              
              $display("Page Received; %h",c72_rx_page);
              
                /* rx page clear
                 */
                /* write rx control register
                 [6]  - enable rx
                 [0]  - clear rdy
                 */
              cpu_write((16'h30<<2),32'h41);
        
              if (c72_rx_page != c72_tx_page)
                begin
                   $display("Error; bad page received");
                   c72_rx_error = 1'b1;
                end
        
                /* increment
                 */
                c72_tx_page  = (c72_tx_page + 32'h12345678);
                c72_tx_cnt   = (c72_tx_cnt + 1'h1);
               end
        
             if (c72_rx_error == 0)
               $display("C72 Passed");
             else
               $display("C72 Failed");
        
             repeat (32)
               @(posedge sys_clk);
          end
        
        if (TB_C73==1)
          begin
             $display;
             $display("PCS Clause 73");
        
           // Set pma_tx_sel to 2
             cpu_write ((16'h80<<2),32'h02); 
             
             c73_rx_error  = 1'b0;
        
           /* write tx page
            */
             c73_tx_page  = 48'h1234_5678_9abc;
             
              for (i=0;i<6;i=i+1)
                cpu_write (((16'h8+i)<<2), {24'b0, c73_tx_page[i*8+:8]});
         
            /* write tx ack register
             [4] - ack on
             [0] - ack cnt
             */
              cpu_write ((16'h1<<2),32'h07);
         
            /* write tx lane register
             [0:3] - lane number
             */
              cpu_write ((16'h3<<2),32'h0);
         
            /* write tx control register
             [6] - enable tx
             [0] - send page (continuously)
             */
              cpu_write ((16'h0<<2),32'h41);
         
            /* write rx lane register
             [0:3] - lane number
             */
              cpu_write ((16'h13<<2),32'h0);
              
            /* write rx control register
             [6]  - enable rx
             */
              cpu_write ((16'h10<<2),32'h40);
         
            /* write rx ack register
             [4]  - include ack in match
             */
              cpu_write ((16'h11<<2),32'h00);
         
            /* wait for page to be received
             */
              $display("Waiting for page to be received");
         
              c73_rx_page_rdy  = 1'b0;
         
              while (!c73_rx_page_rdy)
                begin
                  cpu_read ((16'h10<<2),task_rdata);
                  c73_rx_page_rdy  = task_rdata[0];
                end
         
            /* read page received
             */
              for (i=0;i<6;i=i+1)
                cpu_read (((16'h18+i)<<2),c73_rx_page[i*8+:8]);
         
              $display("Page Received; %h",c73_rx_page);
              
              if (c73_rx_page != c73_tx_page)
                begin
                  $display("Error, bad page");
                  c73_rx_error = 1'b1;
                end
         
            /* rx page clear
             */
            /* write rx control register
             [6]  - enable rx
             [0]  - clear rdy
             */
              cpu_write ((16'h10<<2),32'h41);
             
             if (c73_rx_error == 0)
               $display("C73 Passed");
             else
               $display("C73 Failed");
        
             repeat (32)
               @(posedge sys_clk);
          end
	  end
	  
    if(TYPE_10G == 2) 
      begin
        $display("");
        $display("TestBench (Visual) for emac4_top w/ XGMII \n");
        
        cfg_size_min   = 16'd20;
        cfg_size_max   = 16'd201;
        
        cfg_rep_r      = 16'h1;
        cfg_ipg_r      =  8'd0;
        
        start_strb_r   =  1'b0;
        
        sys_sreset     =  1'b1;
        cpu_sreset    =  1'b1;

        bypass_scrm   =  1'b0;
        
        /* CPU Reset
         */
        repeat (32)
          @(posedge cpu_clk);
          cpu_sreset = 1'b0;
        @(posedge cpu_clk);
        
        /* Sys Reset
        */
        repeat (32)
        @(posedge sys_clk);
        sys_sreset = 1'b0;
        @(posedge sys_clk);
        
        /* Wait
        */
        while (!mtx_rdy)
        @(posedge sys_clk);
		
        // static_mac_tx_ifg_cnt = 12
        // static_mac_tx_ifg_dic_mode_en = 1
        // static_mac_tx_preamble_en = 1
        // static_mac_tx_preamble_wcm1 = 1
        cpu_write ((16'hA0<<2),32'hc19); 

        // cfg_sys_mac_tx_fifo_paf = 8
        // cfg_sys_mac_tx_en = 1518
        // mac_tx_max_pkt_len = 8
        cpu_write ((16'hA3<<2),32'h1105EE); 
		
        // sys_mac_tx_fcs_ins = 1
        // sys_mac_tx_fcs_err = 0
        // sys_mac_tx_fcs_stomp = 0
        // sys_mac_tx_ifg_cnt = 0
        cpu_write ((16'hA4<<2),32'h100); 
		
        /* Wait
         */
        repeat (32)
        @(posedge sys_clk);
        
        /* Start
        */
        start_strb_r  = 1'b1;
        @(posedge sys_clk);
        start_strb_r  = 1'b0;
        @(posedge sys_clk);
        
        $display("Transmitting packets");
        
        while (1'b1)
        @(posedge sys_clk);
      end
    $display;
    $stop;
    $finish;
  end

   
/*************************************************************************************
 * Visual
 * 
 *************************************************************************************/
 generate
   if(TYPE_10G == 1)
     begin
         /* tx
          */
       always @(posedge sys_clk)
         if (!sys_sreset & mtx_first_strb)
           $display("Transmitting packets");
       
       always @(posedge sys_clk)
         if (!sys_sreset & mtx_end_strb)
           $display("Done transmitting packets");
       
         /* rx
          */
       always @(posedge sys_clk)
         if (!sys_sreset & mrx_first_strb)
           $display("Receiving packets");
       
       always @(posedge sys_clk)
         if (!sys_sreset & mrx_end_strb)
           begin
             if (mrx_end_ok)
               begin
                  $display("Done receiving packets");
                  $display("Test Passed");
               end
             else
               begin
                  $display("Done receiving packets");
                  $display("Test Failed");
               end
           end
     end
   endgenerate
   
  generate
    if(TYPE_10G == 2)
      begin
         /* tx
          */
       always @(posedge sys_clk)
         begin
         if (!sys_sreset & mtx_end_strb)
          $display("Done transmitting packets");
         end
       
         /* rx
          */
       always @(posedge sys_clk)
         if (sys_sreset)
           rx_1st_r  <= 1'b1;
         else if (mrx_en)
           rx_1st_r  <= 1'b0;
       
       always @(posedge sys_clk)
         if (!sys_sreset & mrx_en & rx_1st_r)
           $display("Receiving packets");
    
       always @(posedge sys_clk)
         if (!sys_sreset & mrx_end_strb)
           begin
          if (mrx_end_ok)
            begin
               $display("Done receiving packets");
               $display("Test Passed");
            end
          else
            begin
               $display("Done receiving packets");
               $display("Test Failed");
            end
        
          repeat(32)
            @(posedge sys_clk);
            $display;
            $stop;
            $finish;
           end
          end
   endgenerate
/************************************************************************/
// Supporting Tasks

// This task does a cpu write 
task cpu_write;
input [15:0] address;
input [31:0] data;
begin
  $display("CPU Write %04x = %04x",address,data);
  @(negedge PCLK);  
  PWRITE  = 1'b0;
  @(negedge PCLK);  
  PENABLE  = 1'b0;
  PSEL  = 1'b1;
  PADDR = address;
  PWDATA  = data;  
  PWRITE  = 1'b1;
  @(negedge PCLK);  
  PENABLE  = 1'b1;
  @(negedge PCLK);  
  while (PREADY==1'b0) @(negedge PCLK);
  PWRITE  = 1'b0;
  PENABLE  = 1'b0;
  PSEL  = 1'b0;
end
endtask

// This task does a cpu read 
task cpu_read;
input [15:0] address;
output [31:0] data;
reg [31:0] data;
begin
  @(negedge PCLK);  
  PWRITE  = 1'b1;
  @(negedge PCLK);  
  PENABLE  = 1'b0;
  PSEL  = 1'b1;
  PADDR = address;
  PWRITE  = 1'b0;
  @(negedge PCLK);  
  PENABLE  = 1'b1;
  @(negedge PCLK);
  while (PREADY==1'b0) @(negedge PCLK);
  data = PRDATA;
  PWRITE  = 1'b1;
  PENABLE  = 1'b0;
  PSEL  = 1'b0;
  $display("CPU Read %04x = %04x",address,data);
end
endtask

endmodule 

